/*

Miniature Serial I/O package 23.04.2003

OH2NLT Juha Niinikoski

*/

#include	<pic.h>

/* Serial interface setup */

void serial_setup(void)
{
	#define BAUD 9600
	#define DIVIDER ((PIC_CLK/(16UL * BAUD) -1))
	#define HIGH_SPEED 1

	TXEN=0;			//reset transmitter
	CREN=0;			//reset receiver

	SPBRG=DIVIDER;
	BRGH=HIGH_SPEED;	//data rate for sending
	SYNC=0;			//asynchronous
	SPEN=1;			//enable serial port pins
	SREN=0;			//no effect
	TXIE=0;			//disable tx interrupts
	RCIE=0;			//disable rx interrupts
	TX9=0;			//8-bit transmission
	RX9=0;			//8-bit reception

	CREN=1;			//enable reception
	TXEN=1;			//enable the transmitter

}


/* Writes a character to the serial port */

void putch(unsigned char c)
	{
	while(!TXIF)			//set when register is empty
		{
		if (OERR)		// this is neded to prevent UART lockup ?
			{
			TXEN=0;
			CREN=0;
			CREN=1;
			TXEN=1;
			}
		}

	TXREG=c;
	}



/* Gets a character from the serial port without timeout */

unsigned char getch(void)
	{
	unsigned char dummy;
	while(!RCIF)
	{
	if (OERR)			// reset receiver if overrun errors
		{
		TXEN=0;
		TXEN=1;
		CREN=0;
		CREN=1;
		}
	if (FERR)			// clear UART if framing error
		{
		dummy=RCREG;
		TXEN=0;
		TXEN=1;
		}
	}
	return RCREG;
	}

/* Check if character available, but do not read character serial port
   returns 0 if no characters available. Returns 1 if character available. */

unsigned char keypressed(void)
	{
	unsigned char dummy;

	if (OERR)			// reset UART if overrun error
		{
		TXEN=0;
		TXEN=1;
		CREN=0;
		CREN=1;
		return(0);		// if errors return not pressed
		}					
	if (FERR)			// clear UART if framing error
		{
		dummy=RCREG;
		TXEN=0;
		TXEN=1;
		return(0);		// if errors return not pressed
		}

	if(RCIF)			// check if character available
		return(1);
	else
		return(0);
	}



/* Clear serial port receiver */

void clear_usart_input(void)
	{
	unsigned char dummy;

	if (OERR || FERR)		// reset receiver if errors
		{
		TXEN=0;
		CREN=0;
		CREN=1;
		TXEN=1;
		}
	while(RCIF)
		dummy = RCREG;			// carbage input if any
	}


/* Write string */

void putst(register const char *str)
{
	while((*str)!=0)
	{
	putch(*str);
    	if (*str==13) putch(10);
    	if (*str==10) putch(13);
	str++;
	}
}

/* Writes a character to the serial port in hex */

void putchhex(unsigned char c)
{
	unsigned char temp;

	// transmits in hex

	temp=c;

	c=(c >> 4);
	if (c<10) c+=48; else c+=55;
	putch(c);

	c=temp;

	c=(c & 0x0F);
	if (c<10) c+=48; else c+=55;
	putch(c);
}






