/*
 *
 * Simple wait and delay routines using Timer 2
 *
 * OH2NLT Juha Niinikoski 7.11.2002
 *
 *
*/

// delays calculated for 4 MHz clock

#define TICK 2							// 2ms tick (fastest possible for 1,8432MHz XTAL)
static char tick_ctr;					// debounce tick timer

/* Init TMR2 for 1 mS interrupt generation */

void init_timers(void)
	{
	tick_ctr = TICK;

	OPTION = (OPTION & 0xCF);  			// 128 prescaler for WDT, 1MHz clock for TMR0

	PR2 = ((PIC_CLK /16000)-1);			// set TMR2 cycle time 1mS
	T2CKPS0 = 1;						// Prescaler = 4 = 250kHz clock for TMR2
	TMR2ON = 1;							// start TIMER2
	PEIE = 1;							// enable peripheral IRQs
	TMR2IE = 1;							// enable TMR2 IRQ
	GIE = 1;							// enable interrupts
	}


/* IRQ handler */
// 1ms timer
static void interrupt
tm02int()
	{
	TMR2IF = 0;							// reset IRQ flag
	tick_ctr--;
	if(tick_ctr == 0)
		{
		tick_ctr = TICK;
		tick  = 1;						// set event indicator
		}
	}


/* Delays a multiple microseconds + start delay at 4 MHz  TMR0 timer */

void delayu( unsigned char usec )
	{
	TMR0 = 0;
	while ( TMR0 < usec );
	}

#define u250	(PIC_CLK /16000)		// 250 us

/* Millisecond delays */

void delayms( unsigned char millisec )
	{
	char x;
	for(x=0; x<millisec; x++)
		{
		delayu(u250);
		delayu(u250);
		delayu(u250);
		delayu(u250);
		}
	}	


